/* Copyright 2000, Jan D. Wolter and Steven R. Weiss, All Rights Reserved. */

#include "backtalk.h"
#include <ctype.h>

#include "adm_group.h"
#include "sysdict.h"
#include "stack.h"
#include "mod_group.h"


/* FUNC_NEWGROUP:   <groupname> <member array> newgroup <flag>
 *   Creates a new group with the given name and list of members.  Returns
 *   0 on success, 1 if the group already exists.
 */

void func_newgroup()
{
#if NOPWEDIT
    die("Cannot add real unix groups");
#else
Array *members= pop_array();
char *gname= pop_string();
struct grp *groups;
int i;

    if (!bdict(VAR_AMADM))
    	die("Only conference administrator may create new groups");

    groups= load_grps();
    if (add_grp(&groups,gname))
    {
        free_grps(groups);
    	push_integer(1);
    }
    else
    {
    	for (i= 0; i < members->sz; i++)
	{
	    if (type(members->a[i]) != TK_STRING)
	    	die("member array may contain only string values");
	    add_memb(groups, gname, sval(members->a[i]));
	}
        write_grps(groups,1);
    	push_integer(0);
    }
    free_array(members);
    free(gname);
#endif /*NOPWEDIT*/
}


/* FUNC_NEWGROUPMEM:   <gname> <member> newgroupmem <flag>
 *   Add a user to an existing group.  Returns 0 on success, 1 if the group
 *   does not exists.
 */

void func_newgroupmem()
{
#if NOPWEDIT
    die("Cannot add members to real unix groups");
#else
char *mem= pop_string();
char *gname= pop_string();
struct grp *groups;

    if (!bdict(VAR_AMADM))
    	die("Only conference administrator may add users to groups");

    groups= load_grps();
    if (add_memb(groups,gname,mem))
    {
        free_grps(groups);
    	push_integer(1);
    }
    else
    {
        write_grps(groups,1);
    	push_integer(0);
    }
    free(mem);
    free(gname);
#endif /*NOPWEDIT*/
}


/* FUNC_DELGROUP:   <groupname> delgroup <flag>
 *   Creates a new group with the given name and list of members.  Returns
 *   0 on success, 1 if the group did not exist.
 */

void func_delgroup()
{
#if NOPWEDIT
    die("Cannot delete real unix groups");
#else
char *gname= pop_string();
struct grp *groups;

    if (!bdict(VAR_AMADM))
    	die("Only conference administrator may remove groups");

    groups= load_grps();
    if (del_grp(&groups,gname))
    {
        free_grps(groups);
    	push_integer(1);
    }
    else
    {
        write_grps(groups,1);
    	push_integer(0);
    }
    free(gname);
#endif /*NOPWEDIT*/
}


/* FUNC_DELGROUPMEM:   <gname> <member> delgroupmem <flag>
 *   Remove a user from a group.  Returns 0 on success, 1 if the group
 *   does not exists or the user was not a member of it.
 */

void func_delgroupmem()
{
#if NOPWEDIT
    die("Cannot delete members from real unix groups");
#else
char *mem= pop_string();
char *gname= pop_string();
struct grp *groups;

    if (!bdict(VAR_AMADM))
    	die("Only conference administrator may remove users from groups");

    groups= load_grps();
    if (del_memb(groups,gname,mem))
    {
        free_grps(groups);
    	push_integer(1);
    }
    else
    {
        write_grps(groups,1);
    	push_integer(0);
    }
    free(mem);
    free(gname);
#endif /*NOPWEDIT*/
}
